﻿var fileDiff = require('./file-diff.js'),
    glob = require("glob"),
    Promise = require('promise'),
    path = require('path'),
    fs = require('fs'),
    fileSignature = require('./file-signature.js'),
    methodKeyCreator = require('./method-key.js'),
    uuid = require('uuid'),
    statuses = require('../statuses.js'),
    //getTransientMethodName = require('./transient-method-naming.js').getTransientMethodName,
    util = require('util'),
    debuglog = util.debuglog ? util.debuglog('sl') : require('../alt-debug-log.js').debuglog('sl'),
    guessMethodName = require('./guess-method-name.js').guessMethodName;

function generateNewMappingForFile(filename, fullFilename) {
    var fileSig = {
        filename: filename,
        status: statuses.FILE_ADDED, // "removed", "modified", "moved", "unchanged"
        methods: []
    };
    filename = filename.replace(/\\/g, '/');
    var oldContent = fs.readFileSync(fullFilename);
    var sig = fileSignature(filename, oldContent);
    if (!sig || sig.error) {
        fileSig.status = statuses.FILE_CORRUPTED;
        return fileSig;
    }
    fileSig.hash = sig.module.hash;
    
    var mapping = {};
    sig.methods.forEach(function (m) {
        var wholeKey = [methodKeyCreator(m.start, m.name, m.isAnonymous), m.hash, new Date().valueOf()].join('|');
        var newId = uuid(wholeKey);
        fileSig.methods.push({
            id: newId,
            isAnonymous: m.isAnonymous,
            hash: m.hash,
            //snippet: m.snippet,
            position: m.start,
            name: m.name,
            uniqueName: m.uniqueName,
            //transientName: getTransientMethodName(m.name, m.start.join(',')),
            status: statuses.METHOD_ADDED
        })
    });
    return fileSig;
}

function tryResolvingMethodMovements(oldSignature, newSignature) {
    //Work-in-progress: Try to match add/delete method pairs and convert this into a single "changed" event
    /*if (!oldSignature || !oldSignature.methods) return;
    var deletedMethods = newSignature.methods.filter(function (f) { return f.status == statuses.METHOD_DELETED });
    var addedMethods = newSignature.methods.filter(function (f) { return f.status == statuses.METHOD_ADDED });

    //for every deleted method
    deletedMethods.forEach(function (deletedMethod) {
        var oldMethodWithTheSameId = oldSignature.methods.filter(function (f) { return f.id == deletedMethod.id });
        if (oldMethodWithTheSameId.length == 0) return;
        var oldMethod = oldMethodWithTheSameId[0];
        //if (oldMethod.name && oldMethod.name===)
    });*/
}

function javascriptHandler(handlerParams) {
    var opts = handlerParams.buildArguments;
    var previousMapping = handlerParams.previousMapping;
    var finalOutput = handlerParams.finalOutput;
    var diffData = handlerParams.diffData;
    var defaultNodeJsExcludedPaths = "test/**/*.js";
    handlerParams.cfg.nodeJsExcludedPaths = handlerParams.cfg.nodeJsExcludedPaths || defaultNodeJsExcludedPaths;
    var excludedPathsArray = handlerParams.cfg.nodeJsExcludedPaths.split(',').map(function (t) { return t.trim() }).filter(function (tt) { return tt.length > 0 });
    
    var p = new Promise(function (resolve, reject) {
        finalOutput.files = finalOutput.files || [];
        fileMap = {};
        
        //map files to a hash for a quicker lookup
        finalOutput.files.forEach(function (f) {
            fileMap[f.filename] = f;
        });
        
        //map previous mapping lookup to a hash for a quicker lookup
        var previousMappingLookup = {};
        if (previousMapping && previousMapping.files) {
            previousMapping.files.forEach(function (f) {
                previousMappingLookup[f.filename] = f;
            });
        }
        
        //map diff data to a hash for a quicker lookup
        var diffLookup = {};
        if (diffData) {
            debuglog('diffData.length = ' + diffData.length);
            diffData.forEach(function (singleFileDiff) {
                if (singleFileDiff.from != singleFileDiff.to) {
                    console.log('ERROR: rename operations are not supported yet');
                }
                if (singleFileDiff.to == '/dev/null') {
                    //match extension
                    if (path.extname(singleFileDiff.from).toLowerCase() != '.js') return;
                    if (!fileMap[singleFileDiff.from]) {
                        var deletedFileInfo = {
                            filename: singleFileDiff.from,
                            status: statuses.FILE_DELETED,
                            methods: []
                        };
                        
                        if (previousMappingLookup.hasOwnProperty(singleFileDiff.from)) {
                            previousMappingLookup[singleFileDiff.from].methods.forEach(function (deletedMethod) {
                                deletedFileInfo.methods.push({ id: deletedMethod.id, status: statuses.METHOD_DELETED });
                            });
                        }
                        
                        finalOutput.files.push(deletedFileInfo);
                    }
                    return; //No need to map, since the file doesn't exist
                }
                diffLookup[singleFileDiff.to] = singleFileDiff;
            });
        }
        
        var files = glob.sync("**/*.js", 
 {
            cwd: opts.workspacepath,
            ignore: ["node_modules/**/*.js"].concat(excludedPathsArray)
        });
        
        files.forEach(function (filename, fileIdx) {
            var fullFilename = path.resolve(opts.workspacepath, filename);
            if (fileMap[filename]) //It is possible for the same project to be in several technologies, like javascript and nodejs. These work on the same file extension and may cause duplicate mapping for the .js files.
            {
                return; //already added that file, skip it 
            }
            
            console.log('Processing [' + (fileIdx + 1) + '/' + files.length + '] ' + fullFilename);
            if (previousMappingLookup.hasOwnProperty(filename)) {
                
                var prevFileMapping = previousMappingLookup[filename];
                if (handlerParams.buildArguments.scm == "none") {
                    //alternative comparison of new signature to old signature
                    var newSignature = generateNewMappingForFile(filename, fullFilename);
                    fileDiff(prevFileMapping, null, newSignature);
                    //tryResolvingMethodMovements(prevFileMapping, newSignature);
                    finalOutput.files.push(newSignature);
                }
                else {
                    if (diffLookup.hasOwnProperty(filename)) {
                        debuglog('File ' + filename + ' has changed');
                        //file has changed
                        var diffContent = diffLookup[filename];
                        var newSignature = generateNewMappingForFile(filename, fullFilename);
                        if (newSignature.status == statuses.FILE_CORRUPTED) {
                            finalOutput.brokenBuild = true;
                            console.log("File " + filename + " could not be parsed or is corrupted.");
                        }
                        else {
                            debuglog('Diffing contents of ' + filename);
                            fileDiff(prevFileMapping, diffContent, newSignature);
                            finalOutput.files.push(newSignature);
                        }
                    }
                    else {
                        debuglog('File ' + filename + ' has NOT been changed');
                        //file has not changed
                        //Just take the previous mapping, and set file status to "unchanged", and all method status to "unchanged"
                        var newMapping = {
                            filename: filename,
                            status: statuses.FILE_NOT_CHANGED,
                            hash: prevFileMapping.hash,
                            methods: []
                        };
                        
                        prevFileMapping.methods.forEach(function (m) {
                            if (m.status == statuses.METHOD_DELETED) return; //skip removed methods
                            m.status = statuses.METHOD_NOT_CHANGED;
                            newMapping.methods.push(m);
                        });
                        
                        finalOutput.files.push(newMapping);
                    }
                }
            }
            else {
                var fileSig = generateNewMappingForFile(filename, fullFilename);
                finalOutput.files.push(fileSig);
            }
        })
        resolve(true);
    });
    return p;
}

module.exports = javascriptHandler