﻿var Promise = require('promise'),
    exec = require('child_process').exec,
    diffParse = require('parse-diff'),
    logParse = require('./parse-log.js').logParse,
    util = require('util'),
    debuglog = util.debuglog ? util.debuglog('sl') : require('../alt-debug-log.js').debuglog('sl');;;

function Git(cfg) {
    this.getCfg = function () { return cfg; }
}

Git.prototype.getDiff = function getDiff(handlerParams) {
    
    var cfg = this.getCfg();
    var p = new Promise(function (resolve, reject) {
        
        if (!handlerParams.previousMapping || 
            !handlerParams.previousMapping.meta) {
            debuglog('GIT: no previous mapping, skipping diff');
            resolve(null);
            return;
        };
        var diffFrom = handlerParams.previousMapping.meta.commit || handlerParams.previousMapping.meta.build;
        
        try {
            debuglog('GIT: invoking git diff --ignore-space-change ' + diffFrom);
            var stdDiff = exec('git diff --ignore-space-change ' + diffFrom, {
                cwd: handlerParams.buildArguments.workspacepath,
                maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
            }, function (error, stdout, stderr) {
                if (error !== null) {
                    console.error('GIT diff error: ' + error);
                    console.error('stderr: ' + stderr);
                    reject(error);
                    return;
                }
                debuglog('diff data: ' + stdout);
                handlerParams.diffData = diffParse(stdout.toString());
                resolve();
            });
            
        } catch (e) {
            console.error('GIT diff error: ' + e);
            reject(e)
        }
    });
    return p;
}

Git.prototype.detectCommitVersion = function detectCommitVersion(handlerParams) {
    var cfg = this.getCfg();
    var p = new Promise(function (resolve, reject) {
        
        try {
            debuglog('GIT: invoking git log -n 1');
            var cmd = "git log -n 1 --pretty=format:%H";
            var gitLogOutput = exec(cmd, {
                cwd: handlerParams.buildArguments.workspacepath,
                maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
            }, function (error, stdout, stderr) {
                //debuglog('stdout: ' + stdout);
                if (error !== null) {
                    console.error('GIT log error: ' + error);
                    console.error('stderr: ' + stderr);
                    reject(error);
                    return;
                }
                
                var commit = stdout.toString().trim();                
                resolve(commit);
            });
            
        } catch (e) {
            console.error('GIT log error: ' + e);
            reject(e);
        }
    });
    return p;
}

Git.prototype.getBranchHistory = function getBranchHistory(handlerParams) {
    var cfg = this.getCfg();
    var p = new Promise(function (resolve, reject) {
        
        if (handlerParams.previousMapping) {
            resolve(null);
            return;
        }
        
        var historyLength = 100;
        if (cfg.branchHistoryLength) {
            try {
                var v = parseInt(cfg.branchHistoryLength);
                if (v < 1) {
                    throw new Error(); //will be caught below
                }
                else {
                    historyLength = v;
                }
            }
            catch (e) {
                console.error("An invalid value was specified for branchHistoryLength in sealights.json: " + cfg.branchHistoryLength + ". Valid values are positive integers. Defaulting to " + historyLength);
            }
        }
        
        try {
            
            debuglog('GIT: invoking git log to get branch history');
            var gitLogOutput = exec('git log -n '+ historyLength+' --pretty=format:%H ', {
                cwd: handlerParams.buildArguments.workspacepath,
                maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
            }, function (error, stdout, stderr) {
                if (error !== null) {
                    console.error('GIT log error: ' + error);
                    console.error('stderr: ' + stderr);
                    reject(error);
                    return;
                }
                var branchLog = logParse(stdout.toString());
                resolve(branchLog);
            });
            
        } catch (e) {
            console.error('GIT log error: ' + e);
            reject(e);
        }
    });
    return p;
}


module.exports = Git;